/*  Andrew Bunday 2004 ncca

  Special thanks to J.Macey and his graphics lib*/

/*##########################################################################*/


#include <iostream>
#include <vector>
#include <tiffio.h>
#include <string>
#include "GraphicsLib.h"
#include "nucleon.h"
#include "atom.h"
#include "shellBox.h"
#include "controlrod.h"


using namespace std;
using namespace GraphicsLib;

typedef vector<Nucleon> NucleonList;
typedef vector<Nucleon>::iterator  NListItt;

typedef vector<atom> AtomList;
typedef vector<atom>::iterator  AListItt;

typedef vector<controlrod> RodList;
typedef vector<controlrod>::iterator RListItt;

NucleonList Nucleons;
NucleonList NucleonsCopy;
AtomList Atoms;
AtomList AtomsCopy;
RodList Rods;

float frametimer;
int framenumber;

shellBox *atomBox;
shellBox *outerBox;

float frustum[6][4];

int WIDTH = 720;
int HEIGHT = 576;

Camera Cam;

int spinxface = 0 ;
int spinyface = 0 ;
int spinzface = 0 ;
int  origx,origy,origz,RotateXY,RotateXZ=0;
int Rotate;

int atomicmass,atomicnumber, numatom, freeNeutrons;

//-----------------------------------------------------------------------
//-----------------------------------------------------------------------

void InitGL()
{
glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
glShadeModel(GL_SMOOTH);
glEnable(GL_DEPTH_TEST);
glEnable(GL_LIGHT0);
GLfloat AmbColour[]={0.2,0.2,0.2};
glLightModelfv(GL_LIGHT_MODEL_AMBIENT,AmbColour);
//Point3 Eye(15.0f,15.0f,15.0f);
Point3 Eye(0,60.0f,85.0f);
//Point3 Eye(5.0f,5.0f,85.0f);
Point3 Look(0.0f,0.0f,0.0f);
Vector Up(0.0f,1.0f,0.0f);
Cam.setDisplay(WIDTH, HEIGHT);
Cam.set(Eye,Look,Up);
Cam.setShape(45,640.0/480.0,0.5,300);
glEnable(GL_LIGHTING);
glEnable(GL_COLOR_MATERIAL);
glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);
}

//-----------------------------------------------------------------------

int writetiff(char *filename, char *description,int x, int y, int width, int height, int compression)
{
  TIFF *file;
  GLubyte *image, *p;
  int i;

  file = TIFFOpen(filename, "w");
  if (file == NULL) {
    return 1;
  }
  image = (GLubyte *) malloc(width * height * sizeof(GLubyte) * 3);

  /* OpenGL's default 4 byte pack alignment would leave extra bytes at the
     end of each image row so that each full row contained a number of bytes
     divisible by 4.  Ie, an RGB row with 3 pixels and 8-bit componets would
     be laid out like "RGBRGBRGBxxx" where the last three "xxx" bytes exist
     just to pad the row out to 12 bytes (12 is divisible by 4). To make sure
     the rows are packed as tight as possible (no row padding), set the pack
     alignment to 1. */
  glPixelStorei(GL_PACK_ALIGNMENT, 1);

  glReadPixels(x, y, width, height, GL_RGB, GL_UNSIGNED_BYTE, image);
  TIFFSetField(file, TIFFTAG_IMAGEWIDTH, (uint32) width);
  TIFFSetField(file, TIFFTAG_IMAGELENGTH, (uint32) height);
  TIFFSetField(file, TIFFTAG_BITSPERSAMPLE, 8);
  TIFFSetField(file, TIFFTAG_COMPRESSION, compression);
  TIFFSetField(file, TIFFTAG_PHOTOMETRIC, PHOTOMETRIC_RGB);
  TIFFSetField(file, TIFFTAG_SAMPLESPERPIXEL, 3);
  TIFFSetField(file, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
  TIFFSetField(file, TIFFTAG_ROWSPERSTRIP, 1);
  TIFFSetField(file, TIFFTAG_IMAGEDESCRIPTION, description);
  p = image;
  for (i = height - 1; i >= 0; i--) {
    if (TIFFWriteScanline(file, p, i, 0) < 0) {
      free(image);
      TIFFClose(file);
      return 1;
    }
    p += width * sizeof(GLubyte) * 3;
  }
  TIFFClose(file);
  return 0;
}

//------------------------------------------------------------------------

void ExtractFrustum()
{
   float   proj[16];
   float   modl[16];
   float   clip[16];
   float   t;

   /* Get the current PROJECTION matrix from OpenGL */
   glGetFloatv( GL_PROJECTION_MATRIX, proj );

   /* Get the current MODELVIEW matrix from OpenGL */
   glGetFloatv( GL_MODELVIEW_MATRIX, modl );

   /* Combine the two matrices (multiply projection by modelview) */
   clip[ 0] = modl[ 0] * proj[ 0] + modl[ 1] * proj[ 4] + modl[ 2] * proj[ 8] + modl[ 3] * proj[12];
   clip[ 1] = modl[ 0] * proj[ 1] + modl[ 1] * proj[ 5] + modl[ 2] * proj[ 9] + modl[ 3] * proj[13];
   clip[ 2] = modl[ 0] * proj[ 2] + modl[ 1] * proj[ 6] + modl[ 2] * proj[10] + modl[ 3] * proj[14];
   clip[ 3] = modl[ 0] * proj[ 3] + modl[ 1] * proj[ 7] + modl[ 2] * proj[11] + modl[ 3] * proj[15];

   clip[ 4] = modl[ 4] * proj[ 0] + modl[ 5] * proj[ 4] + modl[ 6] * proj[ 8] + modl[ 7] * proj[12];
   clip[ 5] = modl[ 4] * proj[ 1] + modl[ 5] * proj[ 5] + modl[ 6] * proj[ 9] + modl[ 7] * proj[13];
   clip[ 6] = modl[ 4] * proj[ 2] + modl[ 5] * proj[ 6] + modl[ 6] * proj[10] + modl[ 7] * proj[14];
   clip[ 7] = modl[ 4] * proj[ 3] + modl[ 5] * proj[ 7] + modl[ 6] * proj[11] + modl[ 7] * proj[15];

   clip[ 8] = modl[ 8] * proj[ 0] + modl[ 9] * proj[ 4] + modl[10] * proj[ 8] + modl[11] * proj[12];
   clip[ 9] = modl[ 8] * proj[ 1] + modl[ 9] * proj[ 5] + modl[10] * proj[ 9] + modl[11] * proj[13];
   clip[10] = modl[ 8] * proj[ 2] + modl[ 9] * proj[ 6] + modl[10] * proj[10] + modl[11] * proj[14];
   clip[11] = modl[ 8] * proj[ 3] + modl[ 9] * proj[ 7] + modl[10] * proj[11] + modl[11] * proj[15];

   clip[12] = modl[12] * proj[ 0] + modl[13] * proj[ 4] + modl[14] * proj[ 8] + modl[15] * proj[12];
   clip[13] = modl[12] * proj[ 1] + modl[13] * proj[ 5] + modl[14] * proj[ 9] + modl[15] * proj[13];
   clip[14] = modl[12] * proj[ 2] + modl[13] * proj[ 6] + modl[14] * proj[10] + modl[15] * proj[14];
   clip[15] = modl[12] * proj[ 3] + modl[13] * proj[ 7] + modl[14] * proj[11] + modl[15] * proj[15];

   /* Extract the numbers for the RIGHT plane */
   frustum[0][0] = clip[ 3] - clip[ 0];
   frustum[0][1] = clip[ 7] - clip[ 4];
   frustum[0][2] = clip[11] - clip[ 8];
   frustum[0][3] = clip[15] - clip[12];

   /* Normalize the result */
   t = sqrt( frustum[0][0] * frustum[0][0] + frustum[0][1] * frustum[0][1] + frustum[0][2] * frustum[0][2] );
   frustum[0][0] /= t;
   frustum[0][1] /= t;
   frustum[0][2] /= t;
   frustum[0][3] /= t;

   /* Extract the numbers for the LEFT plane */
   frustum[1][0] = clip[ 3] + clip[ 0];
   frustum[1][1] = clip[ 7] + clip[ 4];
   frustum[1][2] = clip[11] + clip[ 8];
   frustum[1][3] = clip[15] + clip[12];

   /* Normalize the result */
   t = sqrt( frustum[1][0] * frustum[1][0] + frustum[1][1] * frustum[1][1] + frustum[1][2] * frustum[1][2] );
   frustum[1][0] /= t;
   frustum[1][1] /= t;
   frustum[1][2] /= t;
   frustum[1][3] /= t;

   /* Extract the BOTTOM plane */
   frustum[2][0] = clip[ 3] + clip[ 1];
   frustum[2][1] = clip[ 7] + clip[ 5];
   frustum[2][2] = clip[11] + clip[ 9];
   frustum[2][3] = clip[15] + clip[13];

   /* Normalize the result */
   t = sqrt( frustum[2][0] * frustum[2][0] + frustum[2][1] * frustum[2][1] + frustum[2][2] * frustum[2][2] );
   frustum[2][0] /= t;
   frustum[2][1] /= t;
   frustum[2][2] /= t;
   frustum[2][3] /= t;

   /* Extract the TOP plane */
   frustum[3][0] = clip[ 3] - clip[ 1];
   frustum[3][1] = clip[ 7] - clip[ 5];
   frustum[3][2] = clip[11] - clip[ 9];
   frustum[3][3] = clip[15] - clip[13];

   /* Normalize the result */
   t = sqrt( frustum[3][0] * frustum[3][0] + frustum[3][1] * frustum[3][1] + frustum[3][2] * frustum[3][2] );
   frustum[3][0] /= t;
   frustum[3][1] /= t;
   frustum[3][2] /= t;
   frustum[3][3] /= t;

   /* Extract the FAR plane */
   frustum[4][0] = clip[ 3] - clip[ 2];
   frustum[4][1] = clip[ 7] - clip[ 6];
   frustum[4][2] = clip[11] - clip[10];
   frustum[4][3] = clip[15] - clip[14];

   /* Normalize the result */
   t = sqrt( frustum[4][0] * frustum[4][0] + frustum[4][1] * frustum[4][1] + frustum[4][2] * frustum[4][2] );
   frustum[4][0] /= t;
   frustum[4][1] /= t;
   frustum[4][2] /= t;
   frustum[4][3] /= t;

   /* Extract the NEAR plane */
   frustum[5][0] = clip[ 3] + clip[ 2];
   frustum[5][1] = clip[ 7] + clip[ 6];
   frustum[5][2] = clip[11] + clip[10];
   frustum[5][3] = clip[15] + clip[14];

   /* Normalize the result */
   t = sqrt( frustum[5][0] * frustum[5][0] + frustum[5][1] * frustum[5][1] + frustum[5][2] * frustum[5][2] );
   frustum[5][0] /= t;
   frustum[5][1] /= t;
   frustum[5][2] /= t;
   frustum[5][3] /= t;
}

//-----------------------------------------------------------------------

bool PointInFrustum( Point3 PIF )
{
   int p;

   for( p = 0; p < 6; p++ )
      if( frustum[p][0] * PIF.x + frustum[p][1] * PIF.y + frustum[p][2] * PIF.z + frustum[p][3] <= 0 )
         return false;
   return true;
}

//-----------------------------------------------------------------------

void display()
{
	NListItt a = Nucleons.begin();
	NListItt end = Nucleons.end();
	Vector d;
	GLfloat  dist;

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glPushMatrix();
		glRotated ( (GLdouble) spinxface, 1.0, 0.0, 0.0 ) ;
		glRotated ( (GLdouble) spinyface, 0.0, 1.0, 0.0 ) ;
		glRotated ( (GLdouble) spinzface, 0.0, 0.0, 1.0 ) ;

		ExtractFrustum();

		glDisable(GL_LIGHTING);
		glPushMatrix();
			outerBox->Draw();
		glPopMatrix();
		glPushMatrix();
			atomBox->Draw();
		glPopMatrix();
		glEnable(GL_LIGHTING);



		for(int i=0;i<8;i++) Rods[i].Draw();

		while(a != end)
		{
			d.set((*a).Pos.x-Cam.Eye.x,(*a).Pos.y-Cam.Eye.y,(*a).Pos.z-Cam.Eye.z);
			dist = d.length();
			dist = floor(dist);
			//dist = 5.0;


			if( PointInFrustum((*a).Pos)==true )
			{
				(*a).Draw((GLfloat) dist);
			}
			++a;
		}



	glPopMatrix();
	glutSwapBuffers();

	if(frametimer+40<glutGet(GLUT_ELAPSED_TIME))
	{
		char buffer[10];
		sprintf(buffer, "%d", framenumber);
		char filename[20] = "Reactor_01.";

		strcat (filename,buffer);
  		strcat (filename,".tif");
		//filename=  "Reactor_01." + buffer + ".tif";
		//cout<<filename<<endl;
		writetiff(filename, "OpenGL Reactor Simulation", 0, 0, WIDTH, HEIGHT, COMPRESSION_NONE);
		framenumber++;
	}
}

//-----------------------------------------------------------------------

void SplittingTheAtom(int Split, int splitter)//point to nuetrons in atoms
{
	if( Atoms[Split].U235 != 1)
	{
		//AListItt at = Atoms.begin();
		//AListItt aend = Atoms.end();
		int n = Atoms.size();
		Point3 Atpos, pos;
		Vector Atdir, dir;
		Colour col;

		int num=0, a, b;

		for(a=0; a < Split; a++) num+=Atoms[a].AtomicMass;

		Atdir.set((RandomNum()),(RandomNum()),(RandomNum()));
		dir.set(0,0,0);
		col.set(1,1,0);
		Atpos =Atoms[Split].Pos;

		int c = 0;

		for(b=num; b <= num+Atoms[Split].AtomicMass-2; b++)
		{

			Nucleons[b].colour.set(1,1,0);

		}

		pos =Nucleons[b].Pos; dir = Nucleons[b].Dir; col = Nucleons[b].colour;

		Nucleons.erase(Nucleons.begin()+b,Nucleons.begin()+b+1);
		Nucleons.push_back(Nucleon(pos,dir,col));


		//Atoms.insert(Atoms.begin(),atom(Atpos,dir,92,235,1));
		Atoms.erase(Atoms.begin()+Split, Atoms.begin()+(Split+1));

		//Atdir.set((RNG::uniform()-0.5f),(RNG::uniform()-0.5f),(RNG::uniform()-0.5f));
		Atoms.insert(Atoms.begin()+Split,atom(Atpos,Atdir,56,144,1));

		Atoms.insert(Atoms.begin()+(Split+1),atom(Atpos,Atdir,36,90,1));
		Atoms[Split+1].Dir.set(-(RandomNum()),-(RandomNum()),-(RandomNum()));
		//Atoms.insert(Atoms.begin(),atom(Atpos,dir,92,235,1));

		Atoms[n-1].AtomicMass++;
		//cout<<Atoms[n-1]AtomicMass++<<endl;
	}
}

//--------------------------------------------------------------------------------

bool SphereSphereCollision(Point3 Pos1, GLfloat Radius1, Point3 Pos2, GLfloat Radius2)
{
// the relative position of the spheres
Vector relPos;
//min an max distances of the spheres
GLfloat dist;
float minDist;
relPos =Pos1-Pos2;
// and the distance
dist=relPos.x * relPos.x + relPos.y * relPos.y + relPos.z * relPos.z;
minDist =Radius1 +Radius2;
// if it is a hit
if(dist <=(minDist * minDist))
  return true;
else
  return false;
}

//--------------------------------------------------------------------------------

void  atomCollisions(void)
{

//NListItt start = Nucleons.begin();
AListItt at = Atoms.begin();
unsigned int endAtoms = Atoms.size();


bool collide;

unsigned int num=0, CurrentA=0, ToCheckA=0;

for(unsigned int a=0; a < endAtoms-1; a++) num+=(at+a)->AtomicMass;

for(unsigned int ToCheck=num; ToCheck<Nucleons.size(); ToCheck++)
	{
	for(unsigned int Current=0; Current<endAtoms-1; Current++)
		{
		if(ToCheck == Current)
			continue;
		else
			{
			collide = SphereSphereCollision(Atoms[Current].Pos,1,
						Nucleons[ToCheck].Pos,0.075);
			if(collide== true)
				{
				/*Spheres[Current].Dir.Reverse();
				Spheres[ToCheck].Dir.Reverse();
				Spheres[Current].Hit=true;
				Spheres[ToCheck].Hit=true;*/
						Vector normalSurface = Nucleons[ToCheck].Pos - Atoms[Current].Pos;
						normalSurface.normalize();

						GLfloat x= 2*( Nucleons[ToCheck].Dir.dot(normalSurface));
						Vector d =normalSurface*x;
						Nucleons[ToCheck].Dir=Nucleons[ToCheck].Dir-d;
						Nucleons[ToCheck].colour.set(0,1,0);

						//cout<<Atoms[endAtoms-1].AtomicMass<<endl;
						//Atoms[endAtoms-1].AtomicMass--;
						cout <<"neutron Atom Hit"<<endl;
						CurrentA = Current;
						ToCheckA = ToCheck;

						SplittingTheAtom(CurrentA,ToCheckA);

						break;
				}

			}
		}
	}

	/*if(collide == true)
		{
		SplittingTheAtom(CurrentA,ToCheckA);
		}*/
}


//--------------------------------------------------------------------------------

void shellCollisions(void)
{

float ext[6];
ext[0]=ext[1]=(outerBox->Height/2.0f);
ext[2]=ext[3]=(outerBox->Width/2.0f);
ext[4]=ext[5]=(outerBox->Depth/2.0f);

Vector p;
GLfloat D;
int num=0;

for(int n=0; n < Atoms.size()-1; n++) num+=Atoms[n].AtomicMass;
for (unsigned long int a=0; a<Nucleons.size(); a++)
{
	p.set(Nucleons[a].Pos.x,Nucleons[a].Pos.y,Nucleons[a].Pos.z);

	for(int i=0; i<6; i++)
    		{
        	        D=outerBox->shellNormals[i].dot(p);
		        D+= 0.075;

			if(D >=ext[i])
            			{
					GLfloat x= 2*( Nucleons[a].Dir.dot((outerBox->shellNormals[i])));
            				Vector d =outerBox->shellNormals[i]*x;
					Nucleons[a].Dir=Nucleons[a].Dir-d;
				}
        	}
}

}

//--------------------------------------------------------------------------------

void hullCollisions(void)
{

float ext[6];
ext[0]=ext[1]=(atomBox->Height/2.0f);
ext[2]=ext[3]=(atomBox->Width/2.0f);
ext[4]=ext[5]=(atomBox->Depth/2.0f);

Vector p;
GLfloat D;
int num =0;

//for(int n=0; n < Split; n++) num+=Atoms[a].AtomicMass;

for (unsigned long int a=0; a<Atoms.size()-1; a++)
{
	num+=Atoms[a].AtomicMass;
	p.set(Atoms[a].Pos.x,Atoms[a].Pos.y,Atoms[a].Pos.z);

	for(int i=0; i<6; i++)
    		{
        	        D=atomBox->shellNormals[i].dot(p);
		        D+= 0.075;

			if(D >=ext[i])
            			{
					Nucleons.erase(Nucleons.begin()+num,Nucleons.begin()+num+Atoms[a].AtomicMass);
					Atoms.erase(Atoms.begin()+a, Atoms.begin()+a+1);
				}
        	}
}

}

//--------------------------------------------------------------------------------

void RodCollision(void)
{
	AListItt At = Atoms.begin();
	int n =  Atoms.size();

	int num=0;


	for(int a=0; a < n-1; a++) num+=(At+a)->AtomicMass;

	if(Atoms[n-1].AtomicMass > 0)
	{
		Vector relPos;
		GLfloat dist;
		float minDist, len;
		Point3 p;

		for (unsigned long int b=num; b<Nucleons.size(); b++)
		{
			for(int c=0; c<8;c++)
			{
				len = Rods[c].Length/2;

				if( Nucleons[b].Pos.y > (Rods[c].centrePos.y -len ) && Nucleons[b].Pos.y < (Rods[c].centrePos.y + len))
					{
					p.set(Rods[c].centrePos.x, Nucleons[b].Pos.y, Rods[c].centrePos.z);
					relPos = p - Nucleons[b].Pos;
					dist=relPos.x * relPos.x + relPos.y * relPos.y + relPos.z * relPos.z;
					minDist =Rods[c].Radius/2 + 0.075;

					if(dist <=(minDist * minDist))
					{
						Nucleons.erase(Nucleons.begin()+b,Nucleons.begin()+(b+1));
						Atoms[n-1].AtomicMass--;
						cout<<Atoms[n-1].AtomicMass<<endl;
					}
				}
			}
		}
	}
}

//--------------------------------------------------------------------------------

void Update(int i)
{
 NListItt start = Nucleons.begin();
 AListItt at = Atoms.begin();
// AListItt endA = Atoms.end();

 int endAtoms = Atoms.size();
 int len;
 Point3 centre;
 Vector dir;

for(len=0; len<endAtoms-1; len++)
{
 	centre = (at+len)->Pos;

 	for(int i=0; i<(at+len)->AtomicMass; i++)
 	{
		(*start).Pulsate(centre);
		(*start).UpdateA(centre, dir);
		++start;
	}

	if(Atoms[len].U235 == 1)
	{
	Atoms[len].Update();
	}
}

for(int i=0; i< (at+len)->AtomicMass; i++)
{
	(*start).UpdateB();
	++start;
}

RodCollision();
shellCollisions();
//hullCollisions();
atomCollisions();

glutPostRedisplay();
glutTimerFunc(1,Update,0);
}

//--------------------------------------------------------------------------

void CreateAtoms()
{
Point3 pos;
Vector dir;
Colour c;

AListItt a = Atoms.begin();
//AListItt endA = Atoms.end();
unsigned int endAtoms =Atoms.size();
unsigned int len=0;
Point3 xyz;

float X,Y,Z,theta,phi;

for(len=0; len<endAtoms-1; len++)
	{
	xyz = (a+len)->Pos;
	for(int i=0; i<atomicmass; i++)	//(a+len)->AtomicMass
		{
		Z = 1*((RNG::uniform()*2)-1);
		theta = asin(Z/1);
		phi = RNG::uniform()*(2*PI);
		Y = 1*cos(theta)*sin(phi);
		X = 1*cos(theta)*cos(phi);

		//pos.set(RNG::uniform()-0.5+xyz.x,RNG::uniform()-0.5+xyz.y,RNG::uniform()-0.5+xyz.z);
		pos.set(X+xyz.x,Y+xyz.y,Z+xyz.z);
		//dir.set(RNG::uniform(),RNG::uniform(),RNG::uniform());
		dir.set(0,0,0);
		if (i<=(atomicnumber))//a+len)->AtomicNumber
			c.set(1,0,0);
		else
			c.set(0,0,1);
		Nucleons.push_back(Nucleon(pos,dir,c));
		NucleonsCopy.push_back(Nucleon(pos,dir,c));
		}
  		//cout<<"atom made"<<endl;
	}


for(int j=0; j < freeNeutrons; j++)
	{
	pos.set(2+(RandomNum()),2+(RandomNum()),2+(RandomNum()));
	//dir.set(RNG::uniform()-0.5f,RNG::uniform()-0.5f,RNG::uniform()-0.5f);
	dir.set(RandomNum(),RandomNum(),RandomNum());

	c.set(1,1,1);
	Nucleons.push_back(Nucleon(pos,dir,c));
	NucleonsCopy.push_back(Nucleon(pos,dir,c));
	//cout<<"neutron made"<<endl;
 }
 //cout<<"atoms and neutrons made"<<endl;
}

//-----------------------------------------------------------------------

void ResetSystem()
{
RNG::setSeedWithTime();

AListItt At = AtomsCopy.begin();
AListItt Atend = AtomsCopy.end();

Point3 atomPos;
Vector dir;
int freeNeutrons;

Nucleons.erase(Nucleons.begin(),Nucleons.end());
Atoms.erase(Atoms.begin(),Atoms.end());

while( At != Atend )
{
	atomPos = (At)->Pos;
	dir = (At)->Dir;
	freeNeutrons = (At)->AtomicMass;
	Atoms.push_back(atom(atomPos,dir,0,freeNeutrons,0));
	At++;
}

CreateAtoms();
}


//-----------------------------------------------------------------------

void AddNeutrons()
{
	Point3 pos(0,0,0);
	Vector dir(RandomNum(),RandomNum(),RandomNum());
	Colour c(1,1,1);
	int n = Atoms.size(), i;
	for (i=0; i<5;i++)
	{
	Atoms[n-1].AtomicMass++;
	Nucleons.push_back(Nucleon(pos,dir,c));
	dir.set(RandomNum(),RandomNum(),RandomNum());
	}
	cout<<Atoms[n-1].AtomicMass<<endl;
}

//-----------------------------------------------------------------------

void AddAtoms()
{
	Point3 pos((RNG::uniform()*2*20)-20,(RNG::uniform()*2*20)-20,(RNG::uniform()*2*20)-20);
	Point3 Npos;
	Vector dir(0,0,0);
	Colour c;
	int n = Atoms.size(), i;
	float X,Y,Z,theta,phi;

	Atoms.insert(Atoms.begin(),(atom(pos,dir,atomicnumber,atomicmass,0)));
	AListItt a = Atoms.begin();
	int atomicmass =(a)->AtomicMass;
	int atomicnumber = (a)->AtomicNumber;

	for(int i=0; i<atomicmass; i++)	//(a+len)->AtomicMass
		{
		Z = 1*((RNG::uniform()*2)-1);
		theta = asin(Z/1);
		phi = RNG::uniform()*(2*PI);
		Y = 1*cos(theta)*sin(phi);
		X = 1*cos(theta)*cos(phi);

		//pos.set(RNG::uniform()-0.5+xyz.x,RNG::uniform()-0.5+xyz.y,RNG::uniform()-0.5+xyz.z);
		Npos.set(X+pos.x,Y+pos.y,Z+pos.z);
		//dir.set(RNG::uniform(),RNG::uniform(),RNG::uniform());
		dir.set(0,0,0);
		if (i<=(atomicnumber))//a+len)->AtomicNumber
			c.set(1,0,0);
		else
			c.set(0,0,1);

		Nucleons.insert(Nucleons.begin(),Nucleon(Npos,dir,c));
		}
  		//cout<<"atom made"<<endl;
}

//-----------------------------------------------------------------------

void KeyPressed(unsigned char ch, int x, int y)
{
	RListItt a = Rods.begin();
	RListItt endA = Rods.end();

	switch (ch)
	{
		case 27 : exit(0); break;
		case ' ' : ResetSystem(); break;
		case 110: AddNeutrons(); break;
		case 78 : AddNeutrons(); break;
		case 109: AddAtoms(); break;
		case 77 : AddAtoms(); break;
		case 106 :
			//cout<<"inserting"<<endl;
			while(a!=endA)
			{
				(*a).insert();
				a++;
			}
			break;
		case 107 :
			//cout<<"inserting"<<endl;
			while(a!=endA)
			{
				(*a).remove();
				a++;
			}break;

	}
	glutPostRedisplay();
}

//-----------------------------------------------------------------------

void SpecialKeyPressed(int key, int x, int y)
{
    switch (key)
    {
    	case GLUT_KEY_LEFT	:
		Cam.slide(0.5,0.0,0.0);
		break;
    	case GLUT_KEY_UP	:
    		Cam.slide(0.0,-0.5,0.0);
		break;
    	case GLUT_KEY_RIGHT	:
		Cam.slide(-0.5,0.0,0.0);
    		break;
	case GLUT_KEY_DOWN	:
    		Cam.slide(0.0,0.5,0.0);
		break;
    	case GLUT_KEY_PAGE_UP	:
    		Cam.slide(0.0,0.0,0.5);
		break;
	case GLUT_KEY_PAGE_DOWN	:
    		Cam.slide(0.0,0.0,-0.5);
		break;
    }
glutPostRedisplay();
}

//-----------------------------------------------------------------------

void motion ( int x, int y )
{
if (Rotate)
 {
	 spinyface = ( spinyface + (x - origx) ) % 360 ;
	 spinxface = ( spinxface + (y - origy) ) % 360 ;
	 origx = x;
	 origy = y;
	 glutPostRedisplay();
 }
}

//-----------------------------------------------------------------------

static void Button(int button, int down, int x, int y)
{
Rotate = 0;
  switch(button)
  {
  case GLUT_LEFT_BUTTON:
	 if (down == GLUT_DOWN)
        {
		origx = x;
		origy = y;
		Rotate = 1;
        }
     else Rotate = 0;
   break;
   }
}



//------------------------------------------------------------------------

int main(int argc, char **argv)
{
	RNG::setSeedWithTime();
	frametimer = glutGet(GLUT_ELAPSED_TIME);
	framenumber = 0001;

	Point3 atomPos;
	Vector dir;
	float depth = 20, radius = 5, length = 30;

	cout <<"Enter number of Atoms to create"<<endl;
	cin >> numatom;
	//cout <<"Enter atomic Mass of atoms"<<endl;
	//cin >> atomicmass;
	atomicmass=235;
	//cout <<"Enter atomic Number of atoms"<<endl;
	//cin >> atomicnumber;
	atomicnumber=92;
	cout <<"Enter number of free neutrons"<<endl;
	cin >> freeNeutrons;

	for (int a=0; a<numatom; a++)
	{
		atomPos.set((RNG::uniform()*2*depth)-depth,(RNG::uniform()*2*depth)-depth,(RNG::uniform()*2*depth)-depth);
		dir.set(0,0,0);
		cout << "atomPos" << atomPos << endl;
		Atoms.push_back(atom(atomPos,dir,atomicnumber,atomicmass,0));
		AtomsCopy.push_back(atom(atomPos,dir,atomicnumber,atomicmass,0));
	}

	atomPos.set(0,0,0);
	dir.set(0,0,0);
	Atoms.push_back(atom(atomPos,dir,0,freeNeutrons,0));
	AtomsCopy.push_back(atom(atomPos,dir,0,freeNeutrons,0));


	CreateAtoms();


	for(int i=0;i<8;i++)
	{
		if(i==0){atomPos.set(22,60,22);}
		else if(i==1){atomPos.set(22,60,-22);}
		else if(i==2){atomPos.set(-22,60,-22);}
		else if(i==3){atomPos.set(-22,60,22);}

		else if(i==4){atomPos.set(22,60,0);}
		else if(i==5){atomPos.set(-22,60,0);}
		else if(i==6){atomPos.set(0,60,22);}
		else if(i==7){atomPos.set(0,60,-22);}

		//cout<<atomPos<<endl;
		Rods.push_back(controlrod(radius, length, atomPos));
	}

	atomBox = new shellBox(2*depth+20,2*depth+20,2*depth+20,atomPos);
	outerBox = new shellBox(3*depth,3*depth,3*depth,atomPos);

	glutInit(&argc, argv);
    	glutInitDisplayMode(GLUT_DEPTH | GLUT_RGBA | GLUT_DOUBLE |GLUT_ALPHA |GLUT_ACCUM);
	glutInitWindowSize(WIDTH, HEIGHT);
    	glutCreateWindow("Simple Nucleons");
	//glutFullScreen();
   	glutDisplayFunc(display);

	glutTimerFunc(2,Update,0);
	glutMouseFunc(Button);
	glutMotionFunc(motion);
	glutKeyboardFunc(KeyPressed);
	glutSpecialFunc(SpecialKeyPressed);

    	InitGL();

	glutMainLoop();

    	return 1;
}

//------------------------------------------------------------------------------
//end of file
